@@ -1,5 +1,6 @@
 from numbers import Number, Integral
-from scipy.special import sph_harm
+import copy
+from scipy.special import sph_harm, erf
 import numpy as np
 import sympy as sp
 from shenfun.optimization.cython import evaluate
@@ -69,7 +70,7 @@ def Basis(N, family='Fourier', bc=None, dtype='d', quad=None, domain=None,
         only for Fourier)
     dealias_direct : bool, optional
         Use 2/3-rule dealiasing (only Fourier)
-    cordinates: 2-tuple (coordinate, position vector), optional
+    coordinates: 2-tuple (coordinate, position vector), optional
         Map for curvilinear coordinatesystem.
         The new coordinate variable in the new coordinate system is the first item.
         Second item is a tuple for the Cartesian position vector as function of the
@@ -89,6 +90,7 @@ def Basis(N, family='Fourier', bc=None, dtype='d', quad=None, domain=None,
     """
     par = {'padding_factor': padding_factor,
            'dealias_direct': dealias_direct,
+           'dtype': dtype,
            'coordinates': coordinates}
     par.update(kw)
     if domain is not None:
@@ -99,6 +101,7 @@ def Basis(N, family='Fourier', bc=None, dtype='d', quad=None, domain=None,
             B = fourier.bases.C2CBasis
         else:
             B = fourier.bases.R2CBasis
+        del par['dtype']
         return B(N, **par)
 
     elif family.lower() in ('chebyshev', 'c'):
@@ -133,6 +136,8 @@ def Basis(N, family='Fourier', bc=None, dtype='d', quad=None, domain=None,
                 B = chebyshev.bases.ShenBiPolarBasis
             elif bc.lower() == 'dirichletneumann':
                 B = chebyshev.bases.DirichletNeumannBasis
+            elif bc.lower() == 'neumanndirichlet':
+                B = chebyshev.bases.NeumannDirichletBasis
 
         else:
             raise NotImplementedError
@@ -173,6 +178,10 @@ def Basis(N, family='Fourier', bc=None, dtype='d', quad=None, domain=None,
                 B = legendre.bases.ShenBiPolarBasis
             elif bc.lower() == 'bipolar0':
                 B = legendre.bases.ShenBiPolar0Basis
+            elif bc.lower() == 'dirichletneumann':
+                B = legendre.bases.DirichletNeumannBasis
+            elif bc.lower() == 'neumanndirichlet':
+                B = legendre.bases.NeumannDirichletBasis
 
         return B(N, **par)
 
@@ -265,41 +274,52 @@ class Expr(object):
     ----------
     basis : :class:`.BasisFunction`
         :class:`.TestFunction`, :class:`.TrialFunction` or :class:`.Function`
-    terms : Numpy array of ndim = 3
-        Describes operations performed in Expr
+    terms : list of list of lists of length dimension
+        Describes the differential operations performed on the basis function
+        in the `Expr`
 
-        - Index 0: Vector component. If Expr is rank = 0, then terms.shape[0] = 1.
-          For vectors it equals ndim
+        The triply nested `terms` lists are such that
 
-        - Index 1: One for each term in the form. For example `div(grad(u))`
-          has three terms in 3D:
+        - the outermost list represents a tensor component. There is one item for
+          each tensor component. If the Expr is a scalar with rank = 0, then
+          len(terms) = 1. For vectors it equals the number of dimensions and
+          for second order tensors it equals ndim**2
+
+        - the second nested list represents the different terms in the form, that
+          may be more than one. For example, the scalar valued `div(grad(u))` has
+          three terms in 3D:
 
         .. math::
 
            \partial^2u/\partial x^2 + \partial^2u/\partial y^2 + \partial^2u/\partial z^2
 
-        - Index 2: The operations stored as an array of length = dim
+        - the last inner list represents the differential operations for each term
+          and each tensor component, stored for each as a list of length = dim
 
         The Expr `div(grad(u))`, where u is a scalar, is as such represented
-        as an array of shape (1, 3, 3), 1 meaning it's a scalar, the first 3
+        as a nested list of shapes (1, 3, 3), 1 meaning it's a scalar, the first 3
         because the Expr consists of the sum of three terms, and the last 3
         because it is 3D. The entire representation is::
 
-           array([[[2, 0, 0],
-                   [0, 2, 0],
-                   [0, 0, 2]]])
+            [[[2, 0, 0],
+              [0, 2, 0],
+              [0, 0, 2]]]
 
         where the first [2, 0, 0] term has two derivatives in first direction
         and none in the others, the second [0, 2, 0] has two derivatives in
-        second direction, etc.
+        second direction, and the last [0, 0, 2] has two derivatives in the
+        last direction and none in the others.
 
-    scales :  Numpy array of shape == terms.shape[:2]
+    scales :  list of lists
         Representing a scalar multiply of each inner product. Note that
-        the scalar can be a function of coordinates (using sympy).
+        the scalar can also be a function of coordinates (using sympy).
+        There is one scale for each term in each tensor component, and as
+        such it is a list of lists.
 
-    indices : Numpy array of shape == terms.shape[:2]
+    indices : list of lists
         Index into MixedTensorProductSpace. Only used when basis of form has
-        rank > 0
+        rank > 0. There is one index for each term in each tensor component,
+        and as such it is a list of lists.
 
     Examples
     --------
@@ -313,16 +333,10 @@ class Expr(object):
     >>> v = TestFunction(T)
     >>> e = div(grad(v))
     >>> e.terms()
-    array([[[2, 0, 0],
-            [0, 2, 0],
-            [0, 0, 2]]])
+    [[[2, 0, 0], [0, 2, 0], [0, 0, 2]]]
     >>> e2 = grad(v)
     >>> e2.terms()
-    array([[[1, 0, 0]],
-    <BLANKLINE>
-           [[0, 1, 0]],
-    <BLANKLINE>
-           [[0, 0, 1]]])
+    [[[1, 0, 0]], [[0, 1, 0]], [[0, 0, 1]]]
 
     Note that `e2` in the example has shape (3, 1, 3). The first 3 because it
     is a vector, the 1 because each vector item contains one term, and the
@@ -330,7 +344,6 @@ class Expr(object):
     """
 
     def __init__(self, basis, terms=None, scales=None, indices=None):
-        #assert isinstance(basis, BasisFunction)
         self._basis = basis
         self._terms = terms
         self._scales = scales
@@ -338,14 +351,14 @@ class Expr(object):
         ndim = self.function_space().dimensions
         if terms is None:
             self._terms = np.zeros((self.function_space().num_components(), 1, ndim),
-                                   dtype=np.int)
+                                   dtype=np.int).tolist()
         if scales is None:
-            self._scales = np.ones((self.function_space().num_components(), 1), dtype=object)
+            self._scales = np.ones((self.function_space().num_components(), 1), dtype=object).tolist()
 
         if indices is None:
-            self._indices = basis.offset()+np.arange(self.function_space().num_components())[:, np.newaxis]
+            self._indices = (basis.offset()+np.arange(self.function_space().num_components())[:, np.newaxis]).tolist()
 
-        assert np.prod(self._scales.shape) == self.num_terms()*self.num_components()
+        #assert np.prod(self._scales.shape) == self.num_terms()*self.num_components()
 
     def basis(self):
         """Return basis of Expr"""
@@ -368,10 +381,6 @@ class Expr(object):
         """Return scales of Expr"""
         return self._scales
 
-    def measures(self):
-        """Return measures of Expr"""
-        return self._measures
-
     @property
     def argument(self):
         """Return argument of Expr's basis"""
@@ -380,19 +389,23 @@ class Expr(object):
     def expr_rank(self):
         """Return rank of Expr"""
         if self.dimensions == 1:
-            assert self._terms.shape[0] < 3
-            return self._terms.shape[0]-1
+            assert len(self._terms) < 3
+            return len(self._terms)-1
 
-        if self._terms.shape[0] == 1:
+        if len(self._terms) == 1:
             return 0
-        if self._terms.shape[0] == self._terms.shape[-1]:
+        if len(self._terms) == len(self._terms[0][0]):
             return 1
-        if self._terms.shape[0] == self._terms.shape[-1]**2:
+        if len(self._terms) == len(self._terms[0][0])**2:
             return 2
 
     @property
     def rank(self):
-        """Return rank of Expr's basis"""
+        """Return rank of Expr's :class:`BasisFunction`"""
+        return self._basis.rank
+
+    def basis_rank(self):
+        """Return rank of Expr's :class:`BasisFunction`"""
         return self._basis.rank
 
     def indices(self):
@@ -401,22 +414,118 @@ class Expr(object):
 
     def num_components(self):
         """Return number of components in Expr"""
-        return self._terms.shape[0]
+        return len(self._terms)
 
     def num_terms(self):
         """Return number of terms in Expr"""
-        return self._terms.shape[1]
+        return [len(terms) for terms in self._terms]
 
     @property
     def dimensions(self):
         """Return ndim of Expr"""
-        return self._terms.shape[2]
+        return len(self._terms[0][0])
 
     def index(self):
         if self.num_components() == 1:
             return self._basis.offset()
         return None
 
+    def latexprint(self, x=None, funcname='u'):
+        s = ""
+        x = 'xyzrst' if x is None else x
+        for i, vec in enumerate(self.terms()):
+            if self.num_components() > 1:
+                s += '\\left( '
+
+            for j, term in enumerate(vec):
+                sc = self.scales()[i][j]
+                k = self.indices()[i][j]
+                if not sc == 1:
+                    scp = sp.latex(sc)
+                    if scp.startswith('-'):
+                        s = s.rstrip('+')
+                    s += scp
+                t = np.array(term)
+                cmp = funcname
+                if self.num_components() > 1:
+                    cmp = funcname + '_{%s}'%(x[k])
+                if np.sum(t) == 0:
+                    s += cmp
+                else:
+                    p = '^'+str(np.sum(t)) if np.sum(t) > 1 else ' '
+                    s += "\\frac{\partial%s%s}"%(p, cmp)
+                    for j, ti in enumerate(t):
+                        if ti > 0:
+                            tt = '^'+str(ti) if ti > 1 else ' '
+                            s += "{\\partial%s%s}"%(tt, x[j])
+                s += '+'
+            if self.num_components() > 1:
+                s = s.rstrip('+')
+                s += "\\right) \\mathbf{b}_{%d}"%(i)
+                s += '+'
+
+        return s.rstrip('+')
+
+    def tosympy(self, basis=None, psi=sp.symbols('x,y,z,r,s', real=True)):
+        """Return self evaluated with a sympy basis
+
+        Parameters
+        ----------
+        basis : sympy Expr or string
+            if sympy Expr, then use this directly
+            if str, then use that as name for a generic sympy Function
+        psi : tuple of sympy Symbols
+
+        Examples
+        --------
+        >>> from shenfun import Basis, TensorProductSpace
+        >>> import sympy as sp
+        >>> theta, r = psi = sp.symbols('x,y', real=True, positive=True)
+        >>> rv = (r*sp.cos(theta), r*sp.sin(theta))
+        >>> F0 = Basis(8, 'F', dtype='d')
+        >>> T0 = Basis(8, 'C')
+        >>> T = TensorProductSpace(comm, (F0, T0), coordinates=(psi, rv))
+        >>> u = TrialFunction(T)
+        >>> ue = r*sp.cos(theta)
+        >>> du = div(grad(u))
+        >>> du.tosympy()
+        Derivative(u(x, y), (y, 2)) + Derivative(u(x, y), y)/y + Derivative(u(x, y), (x, 2))/y**2
+        >>> du.tosympy(basis=ue, psi=psi)
+        0
+
+        """
+        s = sp.S(0)
+        is_vector = self.rank
+        ndim = self.dimensions
+        if basis is None:
+            basis = 'u'
+
+        if isinstance(basis, str):
+            # Create sympy Function
+            st = basis
+            if is_vector:
+                basis = []
+                for i in range(self.dimensions):
+                    basis.append(sp.Function(st+'xyzrs'[i])(*psi[:self.dimensions]))
+            else:
+                basis = [sp.Function(st)(*psi[:self.dimensions])]
+        else:
+            if isinstance(basis, sp.Expr):
+                basis = [basis]
+            assert len(basis) == ndim**self.rank
+
+        for i, vec in enumerate(self.terms()):
+            for j, term in enumerate(vec):
+                sc = self.scales()[i][j]
+                k = self.indices()[i][j]
+                b0 = basis[k]
+                tt = tuple([psi[n] for n, l in enumerate(term) for m in range(l)])
+                if np.sum(term) > 0:
+                    s += sc*sp.diff(b0, *tt)
+                else:
+                    s += sc*b0
+        return s
+
     def eval(self, x, output_array=None):
         """Return expression evaluated on x
 
@@ -429,6 +538,9 @@ class Expr(object):
         from shenfun import MixedTensorProductSpace
         from shenfun.fourier.bases import R2CBasis
 
+        if len(x.shape) == 1: # 1D case
+            x = x[None, :]
+
         V = self.function_space()
         basis = self.basis()
 
@@ -455,8 +567,10 @@ class Expr(object):
                     P = test_sp[axis].evaluate_basis_derivative_all(xx, k=k)
                     if not test_sp[axis].domain_factor() == 1:
                         P *= test_sp[axis].domain_factor()**(k)
-                    M.append(P[..., V.local_slice()[axis]])
-                    if isinstance(test_sp[axis], R2CBasis):
+                    if len(x) > 1:
+                        M.append(P[..., V.local_slice()[axis]])
+
+                    if isinstance(test_sp[axis], R2CBasis) and len(x) > 1:
                         r2c = axis
                         m = test_sp[axis].N//2+1
                         if test_sp[axis].N % 2 == 0:
@@ -467,13 +581,16 @@ class Expr(object):
 
                 bv = basis if basis.rank == 0 else basis[ind[base_j]]
                 work.fill(0)
-                if len(x) == 2:
+                if len(x) == 1:
+                    work = np.dot(P, bv)
+
+                elif len(x) == 2:
                     work = evaluate.evaluate_2D(work, bv, M, r2c, last_conj_index, sl)
 
                 elif len(x) == 3:
                     work = evaluate.evaluate_3D(work, bv, M, r2c, last_conj_index, sl)
 
-                sc = self.scales()[vec, base_j]
+                sc = self.scales()[vec][base_j]
                 if not hasattr(sc, 'free_symbols'):
                     sc = float(sc)
                 else:
@@ -488,59 +605,60 @@ class Expr(object):
         return output_array
 
     def __getitem__(self, i):
-        #assert self.num_components() == self.dim()
         basis = self._basis
-        if self.rank > 0:
+        if basis.function_space().is_composite_space > 0:
             basis = self._basis[i]
         else:
             basis = self._basis
         if self.expr_rank() == 1:
             return Expr(basis,
-                        self._terms[i][np.newaxis, :, :],
-                        self._scales[i][np.newaxis, :],
-                        self._indices[i][np.newaxis, :])
+                        [copy.deepcopy(self._terms[i])],
+                        [copy.deepcopy(self._scales[i])],
+                        [copy.deepcopy(self._indices[i])])
 
         elif self.expr_rank() == 2:
             ndim = self.dimensions
             return Expr(basis,
-                        self._terms[i*ndim:(i+1)*ndim],
-                        self._scales[i*ndim:(i+1)*ndim],
-                        self._indices[i*ndim:(i+1)*ndim])
+                        copy.deepcopy(self._terms[i*ndim:(i+1)*ndim]),
+                        copy.deepcopy(self._scales[i*ndim:(i+1)*ndim]),
+                        copy.deepcopy(self._indices[i*ndim:(i+1)*ndim]))
         else:
             raise NotImplementedError
 
     def __mul__(self, a):
-        sc = self.scales().copy()
+        sc = np.array(self.scales())
         if self.expr_rank() == 0:
-            sc = sc * sp.sympify(a)
+            sc = sc*sp.sympify(a)
 
         else:
             if isinstance(a, tuple):
                 assert len(a) == self.num_components()
                 for i in range(self.num_components()):
-                    sc[i] = sc[i] * sp.sympify(a[i])
+                    sc[i] = sc[i]*sp.sympify(a[i])
 
             else:
-                sc *= sp.sympify(a)
+                sc = sc*sp.sympify(a)
 
-        return Expr(self._basis, self._terms.copy(), sc, self._indices.copy())
+        return Expr(self._basis, copy.deepcopy(self._terms), sc.tolist(), copy.deepcopy(self._indices))
 
     def __rmul__(self, a):
         return self.__mul__(a)
 
     def __imul__(self, a):
-        sc = self.scales()
+        sc = np.array(self.scales())
         if self.expr_rank() == 0:
-            sc *= sp.sympify(a)
+            sc = sc*sp.sympify(a)
 
         else:
             if isinstance(a, tuple):
                 assert len(a) == self.dimensions
                 for i in range(self.dimensions):
-                    sc[i] = sc[i] * sp.sympify(a[i])
+                    sc[i] = sc[i]*sp.sympify(a[i])
 
             else:
-                sc *= sp.sympify(a)
+                sc = sc*sp.sympify(a)
+
+        self._scales = sc.tolist()
 
         return self
 
@@ -549,34 +667,34 @@ class Expr(object):
         if not isinstance(a, Expr):
             a = Expr(a)
         assert self.num_components() == a.num_components()
-        assert self.function_space() == a.function_space()
+        #assert self.function_space() == a.function_space()
         assert self.argument == a.argument
         if id(self._basis) == id(a._basis):
             basis = self._basis
         else:
             assert id(self._basis.base) == id(a._basis.base)
             basis = self._basis.base
-        return Expr(basis,
-                    np.concatenate((self.terms(), a.terms()), axis=1),
-                    np.concatenate((self.scales(), a.scales()), axis=1),
-                    np.concatenate((self.indices(), a.indices()), axis=1))
+
+        # Concatenate terms
+        terms, scales, indices = [], [], []
+        for i in range(self.num_components()):
+            terms.append(self._terms[i] + a.terms()[i])
+            scales.append(self._scales[i] + a.scales()[i])
+            indices.append(self._indices[i] + a.indices()[i])
+
+        return Expr(basis, terms, scales, indices)
 
     def __iadd__(self, a):
         assert isinstance(a, (Expr, BasisFunction))
         if not isinstance(a, Expr):
             a = Expr(a)
         assert self.num_components() == a.num_components()
-        assert self.function_space() == a.function_space()
         assert self.argument == a.argument
-        if id(self._basis) == id(a._basis):
-            basis = self._basis
-        else:
-            assert id(self._basis.base) == id(a._basis.base)
-            basis = self._basis.base
-        self._basis = basis
-        self._terms = np.concatenate((self.terms(), a.terms()), axis=1)
-        self._scales = np.concatenate((self.scales(), a.scales()), axis=1)
-        self._indices = np.concatenate((self.indices(), a.indices()), axis=1)
+        self._basis = self.base
+        for i in range(self.num_components()):
+            self._terms[i] += a.terms()[i]
+            self._scales[i] += a.scales()[i]
+            self._indices[i] += a.indices()[i]
         return self
 
     def __sub__(self, a):
@@ -591,10 +709,15 @@ class Expr(object):
         else:
             assert id(self._basis.base) == id(a._basis.base)
             basis = self._basis.base
-        return Expr(basis,
-                    np.concatenate((self.terms(), a.terms()), axis=1),
-                    np.concatenate((self.scales(), -a.scales()), axis=1),
-                    np.concatenate((self.indices(), a.indices()), axis=1))
+
+        # Concatenate terms
+        terms, scales, indices = [], [], []
+        for i in range(self.num_components()):
+            terms.append(self._terms[i] + a.terms()[i])
+            scales.append(self._scales[i] + (-np.array(a.scales()[i])).tolist())
+            indices.append(self._indices[i] + a.indices()[i])
+
+        return Expr(basis, terms, scales, indices)
 
     def __isub__(self, a):
         assert isinstance(a, (Expr, BasisFunction))
@@ -609,14 +732,70 @@ class Expr(object):
             assert id(self._basis.base) == id(a._basis.base)
             basis = self._basis.base
         self._basis = basis
-        self._terms = np.concatenate((self.terms(), a.terms()), axis=1)
-        self._scales = np.concatenate((self.scales(), -a.scales()), axis=1)
-        self._indices = np.concatenate((self.indices(), a.indices()), axis=1)
+        for i in range(self.num_components()):
+            self._terms[i] += a.terms()[i]
+            self._scales[i] += (-np.array(a.scales()[i])).tolist()
+            self._indices[i] += a.indices()[i]
+
         return self
 
     def __neg__(self):
-        return Expr(self.basis(), self.terms().copy(), -self.scales().copy(),
-                    self.indices().copy())
+        return Expr(self.basis(), copy.deepcopy(self.terms()), (-np.array(self.scales())).tolist(),
+                    copy.deepcopy(self.indices()))
+
+    def simplify(self):
+        """Join terms, that are otherwise equal, using scale"""
+        if np.array(self.num_terms()).prod() == 1:
+            return
+
+        tms = []
+        inds = []
+        scs = []
+        for vec, (terms, indices, scales) in enumerate(zip(self.terms(), self.indices(), self.scales())):
+            tms.append([])
+            inds.append([])
+            scs.append([])
+            for i, (term, ind, sc) in enumerate(zip(terms, indices, scales)):
+                match = []
+                if i > 0:
+                    match = np.where((np.array(term) == np.array(tms[-1])).prod(axis=1) == 1)[0]
+                if len(match) > 0:
+                    k = match[0]
+                    if inds[-1][k] == ind:
+                        scs[-1][k] += sc
+                        if scs[-1][k] == 0: # Remove if scale is zero
+                            scs[-1].pop(k)
+                            tms[-1].pop(k)
+                            inds[-1].pop(k)
+                        continue
+                tms[-1].append(term)
+                inds[-1].append(ind)
+                scs[-1].append(sc)
+        self._terms = tms
+        self._indices = inds
+        self._scales = scs
+        return
+
+    def __eq__(self, a):
+        if self.base != a.base:
+            return False
+        if self.num_components() != a.num_components():
+            return False
+        if not np.all(self.num_terms() == a.num_terms()):
+            return False
+        for s0, q0 in zip(self.scales(), a.scales()):
+            for si, qi in zip(s0, q0):
+                if si != qi:
+                    return False
+        for s0, q0 in zip(self.indices(), a.indices()):
+            for si, qi in zip(s0, q0):
+                if si != qi:
+                    return False
+        for t0, p0 in zip(self.terms(), a.terms()):
+            for ti, pi in zip(t0, p0):
+                if not np.all(ti == pi):
+                    return False
+        return True
 
 
 class BasisFunction(object):
@@ -651,7 +830,6 @@ class BasisFunction(object):
     def expr_rank(self):
         """Return rank of expression involving basis"""
         return Expr(self).expr_rank()
-        #return self.function_space().rank
 
     def function_space(self):
         """Return function space of BasisFunction"""
@@ -694,14 +872,14 @@ class BasisFunction(object):
         return self._offset
 
     def __getitem__(self, i):
-        assert self.rank > 0
+        assert self.function_space().is_composite_space
         basespace = self.basespace
         base = self.base
         space = self._space[i]
         offset = self._offset
         for k in range(i):
             offset += self._space[k].num_components()
-        t0 = BasisFunction(space, i, basespace, offset, base)
+        t0 = self.__class__(space, 0, basespace, offset, base)
         return t0
 
     def __mul__(self, a):
@@ -736,7 +914,7 @@ class TestFunction(BasisFunction):
 
     Parameters
     ----------
-    space: TensorProductSpace
+    space: :class:`TensorProductSpace` or :class:`MixedTensorProductSpace`
     index: int, optional
         Component of basis with rank > 0
     basespace : The base :class:`.MixedTensorProductSpace` if space is a
@@ -750,17 +928,6 @@ class TestFunction(BasisFunction):
     def __init__(self, space, index=0, basespace=None, offset=0, base=None):
         BasisFunction.__init__(self, space, index, basespace, offset, base)
 
-    def __getitem__(self, i):
-        assert self.rank > 0
-        basespace = self.basespace
-        base = self.base
-        space = self._space[i]
-        offset = self._offset
-        for k in range(i):
-            offset += self._space[k].num_components()
-        t0 = TestFunction(space, i, basespace, offset, base)
-        return t0
-
     @property
     def argument(self):
         return 0
@@ -770,7 +937,7 @@ class TrialFunction(BasisFunction):
 
     Parameters
     ----------
-    space: TensorProductSpace
+    space: :class:`TensorProductSpace` or :class:`MixedTensorProductSpace`
     index: int, optional
         Component of basis with rank > 0
     basespace : The base :class:`.MixedTensorProductSpace` if space is a
@@ -783,17 +950,6 @@ class TrialFunction(BasisFunction):
     def __init__(self, space, index=0, basespace=None, offset=0, base=None):
         BasisFunction.__init__(self, space, index, basespace, offset, base)
 
-    def __getitem__(self, i):
-        assert self.rank > 0
-        basespace = self.basespace
-        base = self.base
-        space = self._space[i]
-        offset = self._offset
-        for k in range(i):
-            offset += self._space[k].num_components()
-        t0 = TrialFunction(space, i, basespace, offset, base)
-        return t0
-
     @property
     def argument(self):
         return 1
@@ -813,18 +969,38 @@ class ShenfunBaseArray(DistArray):
             if not space.num_components() == 1:
                 shape = (space.num_components(),) + shape
 
-            if hasattr(buffer, 'free_symbols'):
+            # if a list of sympy expressions
+            if isinstance(buffer, (list, tuple)):
+                assert len(buffer) == len(space.flatten())
+                sympy_buffer = buffer
+                buffer = Array(space)
+                dtype = space.forward.input_array.dtype
+                for i, buf0 in enumerate(sympy_buffer):
+                    if isinstance(buf0, Number):
+                        buffer.v[i] = buf0
+                    elif hasattr(buf0, 'free_symbols'):
+                        x = buf0.free_symbols.pop()
+                        buffer.v[i] = sp.lambdify(x, buf0)(space.mesh()).astype(dtype)
+
+                if cls.__name__ == 'Function':
+                    buf = Function(space)
+                    buf = buffer.forward(buf)
+                    buffer = buf
+
+            elif hasattr(buffer, 'free_symbols'):
                 # Evaluate sympy function on entire mesh
                 x = buffer.free_symbols.pop()
                 buffer = sp.lambdify(x, buffer)
-                buffer = buffer(space.mesh())
+                buf = buffer(space.mesh()).astype(space.forward.input_array.dtype)
+                buffer = Array(space)
+                buffer[:] = buf
                 if cls.__name__ == 'Function':
-                    buf = np.empty_like(space.forward.output_array)
-                    buf = space.forward(buffer, buf)
+                    buf = Function(space)
+                    buf = buffer.forward(buf)
                     buffer = buf
 
             obj = DistArray.__new__(cls, shape, buffer=buffer, dtype=dtype,
-                                    rank=space.rank)
+                                    rank=space.is_composite_space)
             obj._space = space
             obj._offset = 0
             if buffer is None and isinstance(val, Number):
@@ -840,7 +1016,34 @@ class ShenfunBaseArray(DistArray):
             p0 = space.backward.output_pencil
             dtype = space.forward.input_array.dtype
 
-        # Evaluate sympy function on entire mesh
+        # if a list of sympy expressions
+        if isinstance(buffer, (list, tuple)):
+            assert len(buffer) == len(space.flatten())
+            sympy_buffer = buffer
+            buffer = Array(space)
+            dtype = space.forward.input_array.dtype
+            if cls.__name__ == 'Function':
+                buff = Function(space)
+            mesh = space.local_mesh(True)
+            for i, buf0 in enumerate(sympy_buffer):
+                if isinstance(buf0, Number):
+                    buffer.v[i] = buf0
+                elif hasattr(buf0, 'free_symbols'):
+                    sym0 = buf0.free_symbols
+                    m = []
+                    for sym in sym0:
+                        j = 'xyzrs'.index(str(sym))
+                        m.append(mesh[j])
+                    buffer.v[i] = sp.lambdify(sym0, buf0, modules=['numpy', {'cot': cot, 'Ynm': Ynm, 'erf': erf}])(*m).astype(dtype)
+                else:
+                    raise NotImplementedError
+
+            if cls.__name__ == 'Function':
+                buff = Function(space)
+                buff = buffer.forward(buff)
+                buffer = buff
+
+        # if just one sympy expression
         if hasattr(buffer, 'free_symbols'):
             sym0 = buffer.free_symbols
             mesh = space.local_mesh(True)
@@ -849,17 +1052,19 @@ class ShenfunBaseArray(DistArray):
                 j = 'xyzrs'.index(str(sym))
                 m.append(mesh[j])
 
-            buffer = sp.lambdify(sym0, buffer, modules=['numpy', {'cot': cot, 'Ynm': Ynm}])(*m)
+            buf = sp.lambdify(sym0, buffer, modules=['numpy', {'cot': cot, 'Ynm': Ynm, 'erf': erf}])(*m).astype(space.forward.input_array.dtype)
+            buffer = Array(space)
+            buffer[:] = buf
             if cls.__name__ == 'Function':
-                buf = np.empty_like(space.forward.output_array)
-                buf = space.forward(buffer, buf)
+                buf = Function(space)
+                buf = buffer.forward(buf)
                 buffer = buf
 
         global_shape = space.global_shape(forward_output)
         obj = DistArray.__new__(cls, global_shape,
                                 subcomm=p0.subcomm, val=val, dtype=dtype,
                                 buffer=buffer, alignment=p0.axis,
-                                rank=space.rank)
+                                rank=space.is_composite_space)
         obj._space = space
         obj._offset = 0
         return obj
@@ -882,12 +1087,6 @@ class ShenfunBaseArray(DistArray):
         if self.base is None:
             return None
 
-        #if self.base.shape == self.shape:
-        #    return None
-
-        #if self.rank > 0:
-        #    return None
-
         if self.function_space().num_components() > 1:
             return None
 
@@ -1008,6 +1207,22 @@ class Function(ShenfunBaseArray, BasisFunction):
     def __init__(self, space, val=0, buffer=None):
         BasisFunction.__init__(self, space, offset=0)
 
+    def set_boundary_dofs(self):
+        space = self.function_space()
+        if space.is_composite_space:
+            for i, s in enumerate(space.flatten()):
+                bases = s.bases if hasattr(s, 'bases') else [s]
+                for base in bases:
+                    if base.has_nonhomogeneous_bcs:
+                        base.bc.set_boundary_dofs(self.v[i], True)
+
+        else:
+            bases = space.bases if hasattr(space, 'bases') else [space]
+            for base in bases:
+                if base.has_nonhomogeneous_bcs:
+                    base.bc.set_boundary_dofs(self, True)
+        return self
+
     @property
     def forward_output(self):
         return True
@@ -1056,35 +1271,29 @@ class Function(ShenfunBaseArray, BasisFunction):
         """Project Function to orthogonal basis"""
         space = self.function_space()
         if output_array is None:
-            output_array = Function(space)
+            output_array = Function(space.get_orthogonal())
 
         # In case of mixed space make a loop
-        if space.rank > 0:
-            spaces = space.flatten()
-            # output_array will now loop over first index
-        else:
-            spaces = [space]
-            output_array = [output_array]
-            self = [self]
+        if space.num_components() > 1:
+            for x, subfunction in zip(output_array, self):
+                x = subfunction.to_ortho(x)
+            return output_array
 
-        for i, space in enumerate(spaces):
-            if space.dimensions > 1:
-                naxes = space.get_nonperiodic_axes()
-                axis = naxes[0]
-                base = space.bases[axis]
-                if not base.is_orthogonal:
-                    output_array[i] = base.to_ortho(self[i], output_array[i])
-                if len(naxes) > 1:
-                    input_array = np.zeros_like(output_array[i].__array__())
-                    for axis in naxes[1:]:
-                        base = space.bases[axis]
-                        input_array[:] = output_array[i]
-                        if not base.is_orthogonal:
-                            output_array[i] = base.to_ortho(input_array, output_array[i])
-            else:
-                output_array[i] = space.to_ortho(self[i], output_array[i])
-        if isinstance(output_array, list):
-            return output_array[0]
+        if space.dimensions > 1:
+            naxes = space.get_nonperiodic_axes()
+            axis = naxes[0]
+            base = space.bases[axis]
+            if not base.is_orthogonal:
+                output_array = base.to_ortho(self, output_array)
+            if len(naxes) > 1:
+                input_array = np.zeros_like(output_array.v)
+                for axis in naxes[1:]:
+                    base = space.bases[axis]
+                    input_array[:] = output_array
+                    if not base.is_orthogonal:
+                        output_array = base.to_ortho(input_array, output_array)
+        else:
+            output_array = space.to_ortho(self, output_array)
         return output_array
 
     def mask_nyquist(self, mask=None):
