@@ -4,7 +4,6 @@ weighted inner product.
 """
 from numbers import Number
 import numpy as np
-import sympy as sp
 from shenfun.spectralbase import inner_product, SpectralBase, MixedBasis
 from shenfun.matrixbase import TPMatrix
 from shenfun.tensorproductspace import TensorProductSpace, MixedTensorProductSpace
@@ -18,7 +17,7 @@ __all__ = ('inner',)
 
 def inner(expr0, expr1, output_array=None, level=0):
     r"""
-    Return weighted discrete inner product of linear or bilinear form
+    Return (weighted) discrete inner product of linear or bilinear form
 
     .. math::
 
@@ -49,12 +48,13 @@ def inner(expr0, expr1, output_array=None, level=0):
         :class:`.Function`) an Array or a number. With expressions (Expr) on a
         BasisFunction we typically mean terms like div(u) or grad(u), where
         u is any one of the different types of BasisFunction.
-        One of ``expr0`` or ``expr1`` need to be an expression on a
-        TestFunction. If the second then involves a TrialFunction, a matrix is
-        returned. If one of ``expr0``/``expr1`` involves a TestFunction and the
-        other one is an expression on a Function, or a plain Array, then a
-        linear form is assembled and a Function is returned.
-        If either expr0 or expr1 is a number (typically 1) or a tuple of
+        If one of ``expr0``/``expr1`` involves a TestFunction and the other one
+        involves a TrialFunction, then a tensor product matrix (or a list of
+        tensor product matrices) is returned.
+        If one of ``expr0``/``expr1`` involves a TestFunction and the other one
+        involves a Function, or a plain Array, then a linear form is assembled
+        and a Function is returned.
+        If one of ``expr0``/``expr1`` is a number (typically 1), or a tuple of
         numbers for a vector, then the inner product represents a non-weighted
         integral over the domain. If a single number, then the other expression
         must be a scalar Array or a Function. If a tuple of numbers, then the
@@ -186,21 +186,39 @@ def inner(expr0, expr1, output_array=None, level=0):
     else:
         raise RuntimeError
 
-    if test.rank > 0 and test.expr_rank() > 0: # For vector expressions of rank > 0 use recursive algorithm
+    if isinstance(test, BasisFunction):
+        recursive = test.rank > 0
+    elif isinstance(test, Expr):
+        recursive = test.function_space().is_composite_space
+    if isinstance(trial, Array):
+        assert trial.rank == test.rank
+    elif isinstance(trial, BasisFunction):
+        recursive *= (trial.expr_rank() > 0)
 
+    if recursive: # Use recursive algorithm for vector expressions of expr_rank > 0, e.g., inner(v, grad(u))
         if output_array is None and trial.argument == 2:
             output_array = Function(test.function_space())
 
+        gij = test.function_space().coors.get_covariant_metric_tensor()
         if trial.argument == 2:
             # linear form
-            for (te, tr, x) in zip(test, trial, output_array):
-                x = inner(te, tr, output_array=x)
+            w0 = np.zeros_like(output_array[0])
+            for i, (te, x) in enumerate(zip(test, output_array)):
+                for j, tr in enumerate(trial):
+                    if gij[i, j] == 0:
+                        continue
+                    w0.fill(0)
+                    x += inner(te*gij[i, j], tr, output_array=w0)
             return output_array
 
         result = []
-        for te, tr in zip(test, trial):
-            l = inner(te, tr, level=level)
-            result += l if isinstance(l, list) else [l]
+
+        for i, te in enumerate(test):
+            for j, tr in enumerate(trial):
+                if gij[i, j] == 0:
+                    continue
+                l = inner(te*gij[i, j], tr, level=level)
+                result += l if isinstance(l, list) else [l]
         return result[0] if len(result) == 1 else result
 
     if output_array is None and trial.argument == 2:
@@ -217,8 +235,6 @@ def inner(expr0, expr1, output_array=None, level=0):
                 return output_array
             trial = trial.forward()
 
-    # If trial is an Expr with terms, then compute using bilinear form and matvec
-
     assert isinstance(trial, (Expr, BasisFunction))
     assert isinstance(test, (Expr, BasisFunction))
 
@@ -238,45 +254,48 @@ def inner(expr0, expr1, output_array=None, level=0):
         uh = trial.base
 
     A = []
-    for vec, (base_test, base_trial, test_ind, trial_ind) in enumerate(zip(test.terms(), trial.terms(), test.indices(), trial.indices())): # vector/scalar
-        for test_j, b0 in enumerate(base_test):              # second index test
-            for trial_j, b1 in enumerate(base_trial):        # second index trial
-                dV = test_scale[vec, test_j]*trial_scale[vec, trial_j]*testspace.hi.prod()
-                assert len(b0) == len(b1)
-                trial_sp = trialspace
-                if isinstance(trialspace, (MixedTensorProductSpace, MixedBasis)): # could operate on a vector, e.g., div(u), where u is vector
-                    trial_sp = trialspace.flatten()[trial_ind[trial_j]]
-                test_sp = testspace
-                if isinstance(testspace, (MixedTensorProductSpace, MixedBasis)):
-                    test_sp = testspace.flatten()[test_ind[test_j]]
-                has_bcs = False
-                # Check if scale is zero
-                if dV == 0:
-                    continue
+    gij = testspace.coors.get_covariant_metric_tensor()
+    for vec_i, (base_test, test_ind) in enumerate(zip(test.terms(), test.indices())): # vector/scalar
+        for vec_j, (base_trial, trial_ind) in enumerate(zip(trial.terms(), trial.indices())):
+            g = 1 if len(test.terms()) == 1 else gij[vec_i, vec_j]
+            if g == 0:
+                continue
+            for test_j, b0 in enumerate(base_test):              # second index test
+                for trial_j, b1 in enumerate(base_trial):        # second index trial
+                    dV = test_scale[vec_i][test_j]*trial_scale[vec_j][trial_j]*testspace.hi.prod()*g
+                    assert len(b0) == len(b1)
+                    trial_sp = trialspace
+                    if isinstance(trialspace, (MixedTensorProductSpace, MixedBasis)): # could operate on a vector, e.g., div(u), where u is vector
+                        trial_sp = trialspace.flatten()[trial_ind[trial_j]]
+                    test_sp = testspace
+                    if isinstance(testspace, (MixedTensorProductSpace, MixedBasis)):
+                        test_sp = testspace.flatten()[test_ind[test_j]]
+                    has_bcs = False
+                    # Check if scale is zero
+                    if dV == 0:
+                        continue
 
-                for dv in split(dV):
-                    sc = dv['scale']
-                    scb = dv['scale']
-                    M = []
-                    DM = []
-                    for i, (a, b) in enumerate(zip(b0, b1)): # Third index, one inner for each dimension
-                        ts = trial_sp[i]
-                        tt = test_sp[i]
-                        msx = 'xyzrs'[i]
-                        msi = dv[msx]
+                    for dv in split(dV):
+                        sc = dv['coeff']
+                        scb = dv['coeff']
+                        M = []
+                        DM = []
+                        for i, (a, b) in enumerate(zip(b0, b1)): # Third index, one inner for each dimension
+                            ts = trial_sp[i]
+                            tt = test_sp[i]
+                            msx = 'xyzrs'[i]
+                            msi = dv[msx]
 
-                        # assemble inner product
-                        AA = inner_product((tt, a), (ts, b), msi)
-                        M.append(AA)
-                        if not abs(AA.scale-1.) < 1e-8:
-                            sc *= AA.scale
-                            AA.scale = 1.0
+                            # assemble inner product
+                            AA = inner_product((tt, a), (ts, b), msi)
+                            M.append(AA)
+                            if not abs(AA.scale-1.) < 1e-8:
+                                sc *= AA.scale
+                                AA.scale = 1.0
 
-                        if (ts.boundary_condition() == 'Dirichlet' and not ts.family() in ('laguerre', 'hermite') or
-                                (ts.boundary_condition() == 'Biharmonic' and not ts.family() in ('jacobi',))):
-                            if ts.bc.has_nonhomogeneous_bcs():
+                            if ts.has_nonhomogeneous_bcs:
                                 tsc = ts.get_bc_basis()
-                                BB = inner_product((tt, a), (tsc, b))
+                                BB = inner_product((tt, a), (tsc, b), msi)
                                 if not abs(BB.scale-1.) < 1e-8:
                                     scb *= BB.scale
                                     BB.scale = 1.0
@@ -285,25 +304,23 @@ def inner(expr0, expr1, output_array=None, level=0):
                                     has_bcs = True
                             else:
                                 DM.append(AA)
-                        else:
-                            DM.append(AA)
 
-                    sc = tt.broadcast_to_ndims(np.array([sc]))
-                    if len(M) == 1: # 1D case
-                        M[0].global_index = (test_ind[test_j], trial_ind[trial_j])
-                        M[0].scale = sc[0]
-                        M[0].mixedbase = testspace
-                        A.append(M[0])
-                    else:
-                        A.append(TPMatrix(M, test_sp, sc, (test_ind[test_j], trial_ind[trial_j]), testspace))
-                    if has_bcs:
-                        if len(DM) == 1: # 1D case
-                            DM[0].global_index = (test_ind[test_j], trial_ind[trial_j])
-                            DM[0].scale = scb
-                            DM[0].mixedbase = testspace
-                            A.append(DM[0])
+                        sc = tt.broadcast_to_ndims(np.array([sc]))
+                        if len(M) == 1: # 1D case
+                            M[0].global_index = (test_ind[test_j], trial_ind[trial_j])
+                            M[0].scale = sc[0]
+                            M[0].mixedbase = testspace
+                            A.append(M[0])
                         else:
-                            A.append(TPMatrix(DM, test_sp, sc, (test_ind[test_j], trial_ind[trial_j]), testspace))
+                            A.append(TPMatrix(M, test_sp, sc, (test_ind[test_j], trial_ind[trial_j]), testspace))
+                        if has_bcs:
+                            if len(DM) == 1: # 1D case
+                                DM[0].global_index = (test_ind[test_j], trial_ind[trial_j])
+                                DM[0].scale = scb
+                                DM[0].mixedbase = testspace
+                                A.append(DM[0])
+                            else:
+                                A.append(TPMatrix(DM, test_sp, sc, (test_ind[test_j], trial_ind[trial_j]), testspace))
 
     # At this point A contains all matrices of the form. The length of A is
     # the number of inner products. For each index into A there are ndim 1D
@@ -356,9 +373,11 @@ def inner(expr0, expr1, output_array=None, level=0):
 
     A = B
 
+    # Bilinear form, return matrices
     if trial.argument == 1:
         return A[0] if len(A) == 1 else A
 
+    # Linear form, return output_array
     wh = np.zeros_like(output_array)
     for b in A:
         if uh.rank > 0:
