@@ -36,6 +36,7 @@ from scipy.special import eval_jacobi, roots_jacobi #, gamma
 from mpi4py_fft import fftw
 from shenfun.spectralbase import SpectralBase, Transform, islicedict, slicedict
 from shenfun.utilities import inheritdocstrings
+from shenfun.forms.arguments import Function
 
 try:
     import quadpy
@@ -76,20 +77,23 @@ class JacobiBase(SpectralBase):
             Factor for padding backward transforms.
         dealias_direct : bool, optional
             Set upper 1/3 of coefficients to zero before backward transform
-        cordinates: 2-tuple (coordinate, position vector), optional
-        Map for curvilinear coordinatesystem.
-        The new coordinate variable in the new coordinate system is the first item.
-        Second item is a tuple for the Cartesian position vector as function of the
-        new variable in the first tuple. Example::
+        dtype : data-type, optional
+            Type of input data in real physical space. Will be overloaded when
+            basis is part of a :class:`.TensorProductSpace`.
+        coordinates: 2-tuple (coordinate, position vector), optional
+            Map for curvilinear coordinatesystem.
+            The new coordinate variable in the new coordinate system is the first item.
+            Second item is a tuple for the Cartesian position vector as function of the
+            new variable in the first tuple. Example::
 
-            theta = sp.Symbols('x', real=True, positive=True)
-            rv = (sp.cos(theta), sp.sin(theta))
+                theta = sp.Symbols('x', real=True, positive=True)
+                rv = (sp.cos(theta), sp.sin(theta))
 
     """
 
-    def __init__(self, N, quad="JG", alpha=0, beta=0, domain=(-1., 1.),
+    def __init__(self, N, quad="JG", alpha=0, beta=0, domain=(-1., 1.), dtype=np.float,
                  padding_factor=1, dealias_direct=False, coordinates=None):
-        SpectralBase.__init__(self, N, quad=quad, domain=domain,
+        SpectralBase.__init__(self, N, quad=quad, domain=domain, dtype=dtype,
                               padding_factor=padding_factor, dealias_direct=dealias_direct,
                               coordinates=coordinates)
         self.alpha = alpha
@@ -109,6 +113,7 @@ class JacobiBase(SpectralBase):
         return self.__class__(N,
                               quad=self.quad,
                               domain=self.domain,
+                              dtype=self.dtype,
                               padding_factor=self.padding_factor,
                               dealias_direct=self.dealias_direct,
                               coordinates=self.coors.coordinates,
@@ -119,6 +124,7 @@ class JacobiBase(SpectralBase):
         return self.__class__(self.N,
                               quad=self.quad,
                               domain=self.domain,
+                              dtype=self.dtype,
                               padding_factor=padding_factor,
                               dealias_direct=dealias_direct,
                               coordinates=self.coors.coordinates,
@@ -148,8 +154,13 @@ class JacobiBase(SpectralBase):
 
     def jacobi(self, x, alpha, beta, N):
         V = np.zeros((x.shape[0], N))
-        for n in range(N):
-            V[:, n] = eval_jacobi(n, alpha, beta, x)
+        if mode == 'numpy':
+            for n in range(N):
+                V[:, n] = eval_jacobi(n, alpha, beta, x)
+        else:
+            X = sp.symbols('x', real=True)
+            for n in range(N):
+                V[:, n] = sp.lambdify(X, sp.jacobi(n, alpha, beta, X), 'mpmath')(x)
         return V
 
     def derivative_jacobi(self, x, alpha, beta, k=1):
@@ -166,42 +177,6 @@ class JacobiBase(SpectralBase):
     def vandermonde(self, x):
         return self.jacobi(x, self.alpha, self.beta, self.N)
 
-    def evaluate_basis(self, x, i=0, output_array=None):
-        x = np.atleast_1d(x)
-        if output_array is None:
-            output_array = np.zeros(x.shape)
-        output_array = eval_jacobi(i, self.alpha, self.beta, x, out=output_array)
-        return output_array
-
-    def evaluate_basis_derivative_all(self, x=None, k=0, argument=0):
-        # Overload always using numpy mode here
-        if x is None:
-            x = self.mesh(False, False)
-            #x = self.points_and_weights(mode=mode)[0]
-        if x.dtype == 'O':
-            x = np.array(x, dtype=np.float)
-        return self.derivative_jacobi(x, self.alpha, self.beta, k)
-        #V = np.zeros((x.shape[0], self.N))
-        #for i in range(self.N):
-        #    V[:, i] = self.evaluate_basis_derivative(x, i, k, output_array=V[:, i])
-        #return V
-
-    def evaluate_basis_all(self, x=None, argument=0):
-        if x is None:
-            x = self.mesh(False, False)
-        return self.vandermonde(x)
-
-    def evaluate_basis_derivative(self, x=None, i=0, k=0, output_array=None):
-        if x is None:
-            x = self.mesh(False, False)
-            #x = self.points_and_weights(mode='mpmath')[0]
-        dj = np.prod(np.array([i+self.alpha+self.beta+1+j for j in range(k)]))
-        return dj/2**k*eval_jacobi(i-k, self.alpha+k, self.beta+k, x)
-        #X = sp.symbols('x', real=True)
-        #f = sp.jacobi(i, self.alpha, self.beta, X)
-        #output_array[:] = sp.lambdify(X, f.diff(X, k), 'mpmath')(x)
-        #return output_array
-
     def plan(self, shape, axis, dtype, options):
         if isinstance(axis, tuple):
             assert len(axis) == 1
@@ -244,31 +219,25 @@ class Basis(JacobiBase):
             Factor for padding backward transforms.
         dealias_direct : bool, optional
             Set upper 1/3 of coefficients to zero before backward transform
-        cordinates: 2-tuple (coordinate, position vector), optional
-        Map for curvilinear coordinatesystem.
-        The new coordinate variable in the new coordinate system is the first item.
-        Second item is a tuple for the Cartesian position vector as function of the
-        new variable in the first tuple. Example::
+        dtype : data-type, optional
+            Type of input data in real physical space. Will be overloaded when
+            basis is part of a :class:`.TensorProductSpace`.
+        coordinates: 2-tuple (coordinate, position vector), optional
+            Map for curvilinear coordinatesystem.
+            The new coordinate variable in the new coordinate system is the first item.
+            Second item is a tuple for the Cartesian position vector as function of the
+            new variable in the first tuple. Example::
 
-            theta = sp.Symbols('x', real=True, positive=True)
-            rv = (sp.cos(theta), sp.sin(theta))
+                theta = sp.Symbols('x', real=True, positive=True)
+                rv = (sp.cos(theta), sp.sin(theta))
     """
 
     def __init__(self, N, quad="JG", alpha=-0.5, beta=-0.5, domain=(-1., 1.),
-                 padding_factor=1, dealias_direct=False, coordinates=None):
-        JacobiBase.__init__(self, N, quad=quad, alpha=alpha, beta=beta, domain=domain,
+                 dtype=np.float, padding_factor=1, dealias_direct=False, coordinates=None):
+        JacobiBase.__init__(self, N, quad=quad, alpha=alpha, beta=beta, domain=domain, dtype=dtype,
                             padding_factor=padding_factor, dealias_direct=dealias_direct,
                             coordinates=coordinates)
-        self.plan(int(N*padding_factor), 0, np.float, {})
-
-    def get_refined(self, N):
-        return self.__class__(N,
-                              quad=self.quad,
-                              domain=self.domain,
-                              padding_factor=self.padding_factor,
-                              dealias_direct=self.dealias_direct,
-                              coordinates=self.coors.coordinates,
-                              alpha=self.alpha, beta=self.beta)
+        self.plan(int(N*padding_factor), 0, dtype, {})
 
     @property
     def is_orthogonal(self):
@@ -280,6 +249,52 @@ class Basis(JacobiBase):
     def sympy_basis(self, i=0, x=sp.symbols('x', real=True)):
         return sp.jacobi(i, self.alpha, self.beta, x)
 
+    def evaluate_basis(self, x, i=0, output_array=None):
+        x = np.atleast_1d(x)
+        if output_array is None:
+            output_array = np.zeros(x.shape)
+        if mode == 'numpy':
+            output_array = eval_jacobi(i, self.alpha, self.beta, x, out=output_array)
+        else:
+            X = sp.symbols('x', real=True)
+            f = self.sympy_basis(i, X)
+            output_array[:] = sp.lambdify(X, f, 'mpmath')(x)
+        return output_array
+
+    def evaluate_basis_derivative_all(self, x=None, k=0, argument=0):
+        if x is None:
+            x = self.mpmath_points_and_weights(mode=mode)[0]
+        #if x.dtype == 'O':
+        #    x = np.array(x, dtype=self.dtype)
+        if mode == 'numpy':
+            return self.derivative_jacobi(x, self.alpha, self.beta, k)
+        else:
+            V = np.zeros((x.shape[0], self.N))
+            for i in range(self.N):
+                V[:, i] = self.evaluate_basis_derivative(x, i, k, output_array=V[:, i])
+        return V
+
+    def evaluate_basis_all(self, x=None, argument=0):
+        if x is None:
+            x = self.mpmath_points_and_weights()[0]
+        return self.vandermonde(x)
+
+    def evaluate_basis_derivative(self, x=None, i=0, k=0, output_array=None):
+        if x is None:
+            x = self.points_and_weights(mode=mode)[0]
+        #x = np.atleast_1d(x)
+        if output_array is None:
+            output_array = np.zeros(x.shape, dtype=self.dtype)
+
+        if mode == 'numpy':
+            dj = np.prod(np.array([i+self.alpha+self.beta+1+j for j in range(k)]))
+            output_array[:] = dj/2**k*eval_jacobi(i-k, self.alpha+k, self.beta+k, x)
+        else:
+            X = sp.symbols('x', real=True)
+            f = sp.jacobi(i, self.alpha, self.beta, X)
+            output_array[:] = sp.lambdify(X, f.diff(X, k), 'mpmath')(x)
+        return output_array
+
 
 class ShenDirichletBasis(JacobiBase):
     """Jacobi basis for Dirichlet boundary conditions
@@ -301,25 +316,28 @@ class ShenDirichletBasis(JacobiBase):
             Factor for padding backward transforms.
         dealias_direct : bool, optional
             Set upper 1/3 of coefficients to zero before backward transform
-        cordinates: 2-tuple (coordinate, position vector), optional
-        Map for curvilinear coordinatesystem.
-        The new coordinate variable in the new coordinate system is the first item.
-        Second item is a tuple for the Cartesian position vector as function of the
-        new variable in the first tuple. Example::
+        dtype : data-type, optional
+            Type of input data in real physical space. Will be overloaded when
+            basis is part of a :class:`.TensorProductSpace`.
+        coordinates: 2-tuple (coordinate, position vector), optional
+            Map for curvilinear coordinatesystem.
+            The new coordinate variable in the new coordinate system is the first item.
+            Second item is a tuple for the Cartesian position vector as function of the
+            new variable in the first tuple. Example::
 
-            theta = sp.Symbols('x', real=True, positive=True)
-            rv = (sp.cos(theta), sp.sin(theta))
+                theta = sp.Symbols('x', real=True, positive=True)
+                rv = (sp.cos(theta), sp.sin(theta))
 
     """
-    def __init__(self, N, quad='JG', bc=(0, 0), domain=(-1., 1.),
+    def __init__(self, N, quad='JG', bc=(0, 0), domain=(-1., 1.), dtype=np.float,
                  padding_factor=1, dealias_direct=False, coordinates=None):
-        JacobiBase.__init__(self, N, quad=quad, alpha=-1, beta=-1, domain=domain,
+        JacobiBase.__init__(self, N, quad=quad, alpha=-1, beta=-1, domain=domain, dtype=dtype,
                             padding_factor=padding_factor, dealias_direct=dealias_direct,
                             coordinates=coordinates)
         assert bc in ((0, 0), 'Dirichlet')
         from shenfun.tensorproductspace import BoundaryValues
         self.bc = BoundaryValues(self, bc=bc)
-        self.plan(int(N*padding_factor), 0, np.float, {})
+        self.plan(int(N*padding_factor), 0, dtype, {})
 
     @staticmethod
     def boundary_condition():
@@ -329,6 +347,7 @@ class ShenDirichletBasis(JacobiBase):
         return self.__class__(N,
                               quad=self.quad,
                               domain=self.domain,
+                              dtype=self.dtype,
                               padding_factor=self.padding_factor,
                               dealias_direct=self.dealias_direct,
                               coordinates=self.coors.coordinates,
@@ -338,6 +357,7 @@ class ShenDirichletBasis(JacobiBase):
         return self.__class__(self.N,
                               quad=self.quad,
                               domain=self.domain,
+                              dtype=self.dtype,
                               padding_factor=padding_factor,
                               dealias_direct=dealias_direct,
                               coordinates=self.coors.coordinates,
@@ -365,7 +385,7 @@ class ShenDirichletBasis(JacobiBase):
         if x is None:
             x = self.mpmath_points_and_weights()[0]
         if output_array is None:
-            output_array = np.zeros(x.shape)
+            output_array = np.zeros(x.shape, dtype=self.dtype)
         X = sp.symbols('x', real=True)
         f = self.sympy_basis(i, X)
         output_array[:] = sp.lambdify(X, f.diff(X, k), mode)(x)
@@ -374,22 +394,27 @@ class ShenDirichletBasis(JacobiBase):
     def evaluate_basis(self, x, i=0, output_array=None):
         x = np.atleast_1d(x)
         if output_array is None:
-            output_array = np.zeros(x.shape)
-        #output_array = (1-x**2)*eval_jacobi(i, -self.alpha, -self.beta, x, out=output_array)
-        mmode = 'mpmath' if x.dtype == 'O' else 'numpy'
-        X = sp.symbols('x', real=True)
-        f = self.sympy_basis(i, X)
-        #f = (1-X**2)*sp.jacobi(i, 1, 1, X)
-        output_array[:] = sp.lambdify(X, f, mmode)(x)
+            output_array = np.zeros(x.shape, dtype=self.dtype)
+        if mode == 'numpy':
+            output_array = (1-x**2)*eval_jacobi(i, -self.alpha, -self.beta, x, out=output_array)
+        else:
+            X = sp.symbols('x', real=True)
+            f = self.sympy_basis(i, X)
+            output_array[:] = sp.lambdify(X, f, 'mpmath')(x)
         return output_array
 
     def evaluate_basis_all(self, x=None, argument=0):
-        if x is None:
-            x = self.mpmath_points_and_weights()[0]
-        V = np.zeros((x.shape[0], self.N))
-        #V[:, :-2] = self.jacobi(x, -self.alpha, -self.beta, self.N-2)*(1-x**2)[:, np.newaxis]
-        for i in range(self.N-2):
-            V[:, i] = self.evaluate_basis(x, i, output_array=V[:, i])
+        if mode == 'numpy':
+            if x is None:
+                x = self.mesh(False, False)
+            V = np.zeros((x.shape[0], self.N))
+            V[:, :-2] = self.jacobi(x, 1, 1, self.N-2)*(1-x**2)[:, np.newaxis]
+        else:
+            if x is None:
+                x = self.mpmath_points_and_weights()[0]
+            V = np.zeros((x.shape[0], self.N))
+            for i in range(self.N-2):
+                V[:, i] = self.evaluate_basis(x, i, output_array=V[:, i])
         return V
 
     def points_and_weights(self, N=None, map_true_domain=False, weighted=True, **kw):
@@ -417,7 +442,9 @@ class ShenDirichletBasis(JacobiBase):
     def to_ortho(self, input_array, output_array=None):
         assert self.alpha == -1 and self.beta == -1
         if output_array is None:
-            output_array = np.zeros_like(input_array.v)
+            output_array = Function(self.get_orthogonal())
+        else:
+            output_array.fill(0)
         k = self.wavenumbers().astype(np.float)
         s0 = self.sl[slice(0, -2)]
         s1 = self.sl[slice(2, None)]
@@ -427,7 +454,7 @@ class ShenDirichletBasis(JacobiBase):
         return output_array
 
     def get_orthogonal(self):
-        return Basis(self.N, alpha=self.alpha+1, beta=self.beta+1, domain=self.domain, coordinates=self.coors.coordinates)
+        return Basis(self.N, alpha=self.alpha+1, beta=self.beta+1, dtype=self.dtype, domain=self.domain, coordinates=self.coors.coordinates)
 
     def vandermonde_scalar_product(self, input_array, output_array):
         SpectralBase.vandermonde_scalar_product(self, input_array, output_array)
@@ -451,14 +478,17 @@ class ShenBiharmonicBasis(JacobiBase):
             Factor for padding backward transforms.
         dealias_direct : bool, optional
             Set upper 1/3 of coefficients to zero before backward transform
-        cordinates: 2-tuple (coordinate, position vector), optional
-        Map for curvilinear coordinatesystem.
-        The new coordinate variable in the new coordinate system is the first item.
-        Second item is a tuple for the Cartesian position vector as function of the
-        new variable in the first tuple. Example::
+        dtype : data-type, optional
+            Type of input data in real physical space. Will be overloaded when
+            basis is part of a :class:`.TensorProductSpace`.
+        coordinates: 2-tuple (coordinate, position vector), optional
+            Map for curvilinear coordinatesystem.
+            The new coordinate variable in the new coordinate system is the first item.
+            Second item is a tuple for the Cartesian position vector as function of the
+            new variable in the first tuple. Example::
 
-            theta = sp.Symbols('x', real=True, positive=True)
-            rv = (sp.cos(theta), sp.sin(theta))
+                theta = sp.Symbols('x', real=True, positive=True)
+                rv = (sp.cos(theta), sp.sin(theta))
 
     Note
     ----
@@ -466,13 +496,13 @@ class ShenBiharmonicBasis(JacobiBase):
     inner products are computed without weights, for alpha=beta=0.
 
     """
-    def __init__(self, N, quad='JG', bc=(0, 0, 0, 0), domain=(-1., 1.), padding_factor=1,
-                 dealias_direct=False, coordinates=None):
-        JacobiBase.__init__(self, N, quad=quad, alpha=-2, beta=-2, domain=domain,
+    def __init__(self, N, quad='JG', bc=(0, 0, 0, 0), domain=(-1., 1.), dtype=np.float,
+                 padding_factor=1, dealias_direct=False, coordinates=None):
+        JacobiBase.__init__(self, N, quad=quad, alpha=-2, beta=-2, domain=domain, dtype=dtype,
                             padding_factor=padding_factor, dealias_direct=dealias_direct,
                             coordinates=coordinates)
         assert bc in ((0, 0, 0, 0), 'Biharmonic')
-        self.plan(int(N*padding_factor), 0, np.float, {})
+        self.plan(int(N*padding_factor), 0, dtype, {})
 
     @staticmethod
     def boundary_condition():
@@ -496,7 +526,7 @@ class ShenBiharmonicBasis(JacobiBase):
         if x is None:
             x = self.mpmath_points_and_weights()[0]
         if output_array is None:
-            output_array = np.zeros(x.shape)
+            output_array = np.zeros(x.shape, dtype=self.dtype)
         X = sp.symbols('x', real=True)
         f = self.sympy_basis(i, X)
         output_array[:] = sp.lambdify(X, f.diff(X, k), mode)(x)
@@ -506,14 +536,26 @@ class ShenBiharmonicBasis(JacobiBase):
         x = np.atleast_1d(x)
         if output_array is None:
             output_array = np.zeros(x.shape)
-        output_array[:] = (1-x**2)**2*eval_jacobi(i, 2, 2, x, out=output_array)
+        if mode == 'numpy':
+            output_array[:] = (1-x**2)**2*eval_jacobi(i, 2, 2, x, out=output_array)
+        else:
+            X = sp.symbols('x', real=True)
+            f = self.sympy_basis(i, X)
+            output_array[:] = sp.lambdify(X, f, 'mpmath')(x)
         return output_array
 
     def evaluate_basis_all(self, x=None, argument=0):
-        if x is None:
-            x = self.mesh(False, False)
-        V = np.zeros((x.shape[0], self.N))
-        V[:, :-4] = self.jacobi(x, 2, 2, self.N-4)*((1-x**2)**2)[:, np.newaxis]
+        if mode == 'numpy':
+            if x is None:
+                x = self.mesh(False, False)
+            V = np.zeros((x.shape[0], self.N))
+            V[:, :-4] = self.jacobi(x, 2, 2, self.N-4)*((1-x**2)**2)[:, np.newaxis]
+        else:
+            if x is None:
+                x = self.mpmath_points_and_weights()[0]
+            V = np.zeros((x.shape[0], self.N))
+            for i in range(self.N-4):
+                V[:, i] = self.evaluate_basis(x, i, output_array=V[:, i])
         return V
 
     def vandermonde_scalar_product(self, input_array, output_array):
@@ -530,7 +572,7 @@ class ShenBiharmonicBasis(JacobiBase):
         return points, weights
 
     def mpmath_points_and_weights(self, N=None, map_true_domain=False, weighted=True, **kw):
-        if mode == 'numpy' and not has_quadpy:
+        if mode == 'numpy' or not has_quadpy:
             return self.points_and_weights(N=N, map_true_domain=map_true_domain, weighted=weighted, **kw)
         if N is None:
             N = self.N
@@ -544,7 +586,9 @@ class ShenBiharmonicBasis(JacobiBase):
 
     def to_ortho(self, input_array, output_array=None):
         if output_array is None:
-            output_array = np.zeros_like(input_array.v)
+            output_array = Function(self.get_orthogonal())
+        else:
+            output_array.fill(0)
         k = self.wavenumbers().astype(np.float)
         _factor0 = 4*(k+2)*(k+1)/(2*k+5)/(2*k+3)
         _factor1 = (-2*(2*k+5)/(2*k+7))
@@ -557,7 +601,7 @@ class ShenBiharmonicBasis(JacobiBase):
         return output_array
 
     def get_orthogonal(self):
-        return Basis(self.N, alpha=0, beta=0, domain=self.domain, coordinates=self.coors.coordinates)
+        return Basis(self.N, alpha=0, beta=0, dtype=self.dtype, domain=self.domain, coordinates=self.coors.coordinates)
 
 
 class ShenOrder6Basis(JacobiBase):
@@ -578,14 +622,17 @@ class ShenOrder6Basis(JacobiBase):
             Factor for padding backward transforms.
         dealias_direct : bool, optional
             Set upper 1/3 of coefficients to zero before backward transform
-        cordinates: 2-tuple (coordinate, position vector), optional
-        Map for curvilinear coordinatesystem.
-        The new coordinate variable in the new coordinate system is the first item.
-        Second item is a tuple for the Cartesian position vector as function of the
-        new variable in the first tuple. Example::
+        dtype : data-type, optional
+            Type of input data in real physical space. Will be overloaded when
+            basis is part of a :class:`.TensorProductSpace`.
+        coordinates: 2-tuple (coordinate, position vector), optional
+            Map for curvilinear coordinatesystem.
+            The new coordinate variable in the new coordinate system is the first item.
+            Second item is a tuple for the Cartesian position vector as function of the
+            new variable in the first tuple. Example::
 
-            theta = sp.Symbols('x', real=True, positive=True)
-            rv = (sp.cos(theta), sp.sin(theta))
+                theta = sp.Symbols('x', real=True, positive=True)
+                rv = (sp.cos(theta), sp.sin(theta))
 
     Note
     ----
@@ -593,12 +640,12 @@ class ShenOrder6Basis(JacobiBase):
     inner products are computed without weights, for alpha=beta=0.
 
     """
-    def __init__(self, N, quad='JG', domain=(-1., 1.), padding_factor=1, dealias_direct=False,
+    def __init__(self, N, quad='JG', domain=(-1., 1.), dtype=np.float, padding_factor=1, dealias_direct=False,
                  coordinates=None):
-        JacobiBase.__init__(self, N, quad=quad, alpha=-3, beta=-3, domain=domain,
+        JacobiBase.__init__(self, N, quad=quad, alpha=-3, beta=-3, domain=domain, dtype=dtype,
                             padding_factor=padding_factor, dealias_direct=dealias_direct,
                             coordinates=coordinates)
-        self.plan(int(N*padding_factor), 0, np.float, {})
+        self.plan(int(N*padding_factor), 0, dtype, {})
 
     @staticmethod
     def boundary_condition():
@@ -632,14 +679,26 @@ class ShenOrder6Basis(JacobiBase):
         x = np.atleast_1d(x)
         if output_array is None:
             output_array = np.zeros(x.shape)
-        output_array[:] = (1-x**2)**3*eval_jacobi(i, 3, 3, x, out=output_array)
+        if mode == 'numpy':
+            output_array[:] = (1-x**2)**3*eval_jacobi(i, 3, 3, x, out=output_array)
+        else:
+            X = sp.symbols('x', real=True)
+            f = self.sympy_basis(i, X)
+            output_array[:] = sp.lambdify(X, f, 'mpmath')(x)
         return output_array
 
     def evaluate_basis_all(self, x=None, argument=0):
-        if x is None:
-            x = self.mesh(False, False)
-        V = np.zeros((x.shape[0], self.N))
-        V[:, :-6] = self.jacobi(x, 3, 3, self.N-6)*((1-x**2)**3)[:, np.newaxis]
+        if mode == 'numpy':
+            if x is None:
+                x = self.mesh(False, False)
+            V = np.zeros((x.shape[0], self.N))
+            V[:, :-6] = self.jacobi(x, 3, 3, self.N-6)*((1-x**2)**3)[:, np.newaxis]
+        else:
+            if x is None:
+                x = self.mpmath_points_and_weights()[0]
+            V = np.zeros((x.shape[0], self.N))
+            for i in range(self.N-6):
+                V[:, i] = self.evaluate_basis(x, i, output_array=V[:, i])
         return V
 
     def points_and_weights(self, N=None, map_true_domain=False, weighted=True, **kw):
@@ -665,7 +724,7 @@ class ShenOrder6Basis(JacobiBase):
         return points, weights
 
     def get_orthogonal(self):
-        return Basis(self.N, alpha=0, beta=0, domain=self.domain, coordinates=self.coors.coordinates)
+        return Basis(self.N, alpha=0, beta=0, dtype=self.dtype, domain=self.domain, coordinates=self.coors.coordinates)
 
     def vandermonde_scalar_product(self, input_array, output_array):
         SpectralBase.vandermonde_scalar_product(self, input_array, output_array)
