@@ -6,13 +6,14 @@ from scipy.special import eval_laguerre
 from mpi4py_fft import fftw
 from shenfun.spectralbase import SpectralBase, work, Transform, islicedict, slicedict
 from shenfun.utilities import inheritdocstrings
+from shenfun.forms.arguments import Function
 
 #pylint: disable=method-hidden,no-else-return,not-callable,abstract-method,no-member,cyclic-import
 
 
 @inheritdocstrings
 class LaguerreBase(SpectralBase):
-    """Base class for all Laguerre bases
+    r"""Base class for all Laguerre bases
 
     Parameters
     ----------
@@ -27,14 +28,17 @@ class LaguerreBase(SpectralBase):
             Factor for padding backward transforms.
         dealias_direct : bool, optional
             Set upper 1/3 of coefficients to zero before backward transform
-        cordinates: 2-tuple (coordinate, position vector), optional
-        Map for curvilinear coordinatesystem.
-        The new coordinate variable in the new coordinate system is the first item.
-        Second item is a tuple for the Cartesian position vector as function of the
-        new variable in the first tuple. Example::
+        dtype : data-type, optional
+            Type of input data in real physical space. Will be overloaded when
+            basis is part of a :class:`.TensorProductSpace`.
+        coordinates: 2-tuple (coordinate, position vector), optional
+            Map for curvilinear coordinatesystem.
+            The new coordinate variable in the new coordinate system is the first item.
+            Second item is a tuple for the Cartesian position vector as function of the
+            new variable in the first tuple. Example::
 
-            theta = sp.Symbols('x', real=True, positive=True)
-            rv = (sp.cos(theta), sp.sin(theta))
+                theta = sp.Symbols('x', real=True, positive=True)
+                rv = (sp.cos(theta), sp.sin(theta))
     Note
     ----
     We are using Laguerre functions and not the regular Laguerre polynomials
@@ -46,11 +50,9 @@ class LaguerreBase(SpectralBase):
 
     where :math:`L_k` and :math:`P_k` are the Laguerre function and Laguerre
     polynomials of order k, respectively.
-
     """
-
-    def __init__(self, N, quad="LG", padding_factor=1, dealias_direct=False, coordinates=None):
-        SpectralBase.__init__(self, N, quad=quad, domain=(0., np.inf),
+    def __init__(self, N, quad="LG", dtype=np.float, padding_factor=1, dealias_direct=False, coordinates=None):
+        SpectralBase.__init__(self, N, quad=quad, domain=(0., np.inf), dtype=dtype,
                               padding_factor=padding_factor, dealias_direct=dealias_direct,
                               coordinates=coordinates)
         self.forward = functools.partial(self.forward, fast_transform=False)
@@ -154,6 +156,7 @@ class LaguerreBase(SpectralBase):
     def get_refined(self, N):
         return self.__class__(N,
                               quad=self.quad,
+                              dtype=self.dtype,
                               padding_factor=self.padding_factor,
                               dealias_direct=self.dealias_direct,
                               coordinates=self.coors.coordinates)
@@ -161,6 +164,7 @@ class LaguerreBase(SpectralBase):
     def get_dealiased(self, padding_factor=1.5, dealias_direct=False):
         return self.__class__(self.N,
                               quad=self.quad,
+                              dtype=self.dtype,
                               padding_factor=padding_factor,
                               dealias_direct=dealias_direct,
                               coordinates=self.coors.coordinates)
@@ -181,14 +185,17 @@ class Basis(LaguerreBase):
             Factor for padding backward transforms.
         dealias_direct : bool, optional
             Set upper 1/3 of coefficients to zero before backward transform
-        cordinates: 2-tuple (coordinate, position vector), optional
-        Map for curvilinear coordinatesystem.
-        The new coordinate variable in the new coordinate system is the first item.
-        Second item is a tuple for the Cartesian position vector as function of the
-        new variable in the first tuple. Example::
+        dtype : data-type, optional
+            Type of input data in real physical space. Will be overloaded when
+            basis is part of a :class:`.TensorProductSpace`.
+        coordinates: 2-tuple (coordinate, position vector), optional
+            Map for curvilinear coordinatesystem.
+            The new coordinate variable in the new coordinate system is the first item.
+            Second item is a tuple for the Cartesian position vector as function of the
+            new variable in the first tuple. Example::
 
-            theta = sp.Symbols('x', real=True, positive=True)
-            rv = (sp.cos(theta), sp.sin(theta))
+                theta = sp.Symbols('x', real=True, positive=True)
+                rv = (sp.cos(theta), sp.sin(theta))
 
     Note
     ----
@@ -203,15 +210,15 @@ class Basis(LaguerreBase):
     polynomials of order k, respectively.
     """
 
-    def __init__(self, N, quad="LG", padding_factor=1, dealias_direct=False, coordinates=None):
-        LaguerreBase.__init__(self, N, quad=quad, padding_factor=padding_factor,
+    def __init__(self, N, quad="LG", dtype=np.float, padding_factor=1, dealias_direct=False, coordinates=None):
+        LaguerreBase.__init__(self, N, quad=quad, dtype=dtype, padding_factor=padding_factor,
                               dealias_direct=dealias_direct, coordinates=coordinates)
-        self.plan(int(N*padding_factor), 0, np.float, {})
+        self.plan(int(N*padding_factor), 0, dtype, {})
 
     def eval(self, x, u, output_array=None):
         x = np.atleast_1d(x)
         if output_array is None:
-            output_array = np.zeros(x.shape)
+            output_array = np.zeros(x.shape, dtype=self.dtype)
         output_array[:] = lag.lagval(x, u)*np.exp(-x/2)
         return output_array
 
@@ -241,22 +248,25 @@ class ShenDirichletBasis(LaguerreBase):
             Factor for padding backward transforms.
         dealias_direct : bool, optional
             Set upper 1/3 of coefficients to zero before backward transform
-        cordinates: 2-tuple (coordinate, position vector), optional
-        Map for curvilinear coordinatesystem.
-        The new coordinate variable in the new coordinate system is the first item.
-        Second item is a tuple for the Cartesian position vector as function of the
-        new variable in the first tuple. Example::
+        dtype : data-type, optional
+            Type of input data in real physical space. Will be overloaded when
+            basis is part of a :class:`.TensorProductSpace`.
+        coordinates: 2-tuple (coordinate, position vector), optional
+            Map for curvilinear coordinatesystem.
+            The new coordinate variable in the new coordinate system is the first item.
+            Second item is a tuple for the Cartesian position vector as function of the
+            new variable in the first tuple. Example::
 
-            theta = sp.Symbols('x', real=True, positive=True)
-            rv = (sp.cos(theta), sp.sin(theta))
+                theta = sp.Symbols('x', real=True, positive=True)
+                rv = (sp.cos(theta), sp.sin(theta))
 
     """
-    def __init__(self, N, quad="LG", bc=(0., 0.), padding_factor=1,
+    def __init__(self, N, quad="LG", bc=(0., 0.), dtype=np.float, padding_factor=1,
                  dealias_direct=False, coordinates=None):
-        LaguerreBase.__init__(self, N, quad=quad, padding_factor=padding_factor,
+        LaguerreBase.__init__(self, N, dtype=dtype, quad=quad, padding_factor=padding_factor,
                               dealias_direct=dealias_direct, coordinates=coordinates)
         self.LT = Basis(N, quad)
-        self.plan(int(N*padding_factor), 0, np.float, {})
+        self.plan(int(N*padding_factor), 0, dtype, {})
 
     @staticmethod
     def boundary_condition():
@@ -270,7 +280,9 @@ class ShenDirichletBasis(LaguerreBase):
 
     def to_ortho(self, input_array, output_array=None):
         if output_array is None:
-            output_array = np.zeros_like(input_array.__array__())
+            output_array = Function(self.get_orthogonal())
+        else:
+            output_array.fill(0)
         s0 = self.sl[slice(0, -1)]
         s1 = self.sl[slice(1, None)]
         output_array[s0] = input_array[s0]
@@ -294,7 +306,7 @@ class ShenDirichletBasis(LaguerreBase):
     def eval(self, x, u, output_array=None):
         x = np.atleast_1d(x)
         if output_array is None:
-            output_array = np.zeros(x.shape)
+            output_array = np.zeros(x.shape, dtype=self.dtype)
         w_hat = work[(u, 0, True)]
         w_hat[1:] = u[:-1]
         output_array[:] = lag.lagval(x, u) - lag.lagval(x, w_hat)
@@ -321,4 +333,4 @@ class ShenDirichletBasis(LaguerreBase):
         self.sl = slicedict(axis=self.axis, dimensions=self.dimensions)
 
     def get_orthogonal(self):
-        return Basis(self.N, quad=self.quad, coordinates=self.coors.coordinates)
+        return Basis(self.N, quad=self.quad, dtype=self.dtype, coordinates=self.coors.coordinates)
