@@ -20,28 +20,18 @@ class TDMA(la_TDMA):
     """
     def __call__(self, b, u=None, axis=0):
 
-        v = self.mat.testfunction[0]
-        bc = v.bc
-
         if u is None:
             u = b
         else:
             assert u.shape == b.shape
             u[:] = b[:]
 
-        if v.is_scaled():
-            bc.apply_before(u, False, scales=(-1./np.sqrt(6.), -1./3./np.sqrt(10.)))
-        else:
-            bc.apply_before(u, False, scales=(-1., -1./3.))
-
         if not self.dd.shape[0] == self.mat.shape[0]:
             self.init()
 
         self.TDMA_SymSolve(self.dd, self.ud, self.L, u, axis=axis)
 
-        bc.apply_after(u, False)
-
-        if not self.mat.scale in (1, 1.0):
+        if self.mat.scale not in (1, 1.0):
             u /= self.mat.scale
         return u
 
@@ -138,19 +128,28 @@ class Helmholtz(object):
 
     """
 
-    def __init__(self, *args):
+    def __init__(self, *args, **kwargs):
+        local_shape = kwargs.get('local_shape', None)
+
+        args = list(args)
+        for i, arg in enumerate(args):
+            if hasattr(arg, 'is_bc_matrix'):
+                if arg.is_bc_matrix():
+                    args.pop(i)
+                    break
 
         A, B = args[0], args[1]
         M = {d.get_key(): d for d in (A, B)}
         self.A = A = M.get('ADDmat', M.get('ANNmat'))
         self.B = B = M.get('BDDmat', M.get('BNNmat'))
-
         if len(args) == 2:
             A_scale = self.A.scale
             B_scale = self.B.scale
             if isinstance(self.A, TPMatrix):
                 A = self.A.pmat
                 B = self.B.pmat
+                A_scale *= self.A.pmat.scale
+                B_scale *= self.B.pmat.scale
         else:
             A_scale = args[2]
             B_scale = args[3]
@@ -165,8 +164,11 @@ class Helmholtz(object):
         self.axis = A.axis
         shape = [1]
         T = A.tensorproductspace
-        if T is not None:
-            shape = list(T.local_shape(True))
+        if local_shape is not None:
+            shape = list(local_shape)
+            shape[A.axis] = 1
+        elif T is not None:
+            shape = list(T.shape(True))
             shape[A.axis] = 1
 
         if np.ndim(B_scale) > 1:
@@ -240,7 +242,7 @@ class Helmholtz(object):
         self.TDMA_SymSolve_VC(self.d0, self.d1, self.L, u, self.axis)
 
         if not self.neumann:
-            self.bc.apply_after(u, True)
+            self.bc.set_boundary_dofs(u, True)
 
         return u
 
@@ -346,7 +348,7 @@ class Biharmonic(object):
         S, A, B = args[0], args[1], args[2]
         M = {d.get_key(): d for d in (S, A, B)}
         self.S = M['SBBmat']
-        self.A = M['PBBmat']  # Legendre and Chebyshev differ
+        self.A = M['ABBmat']
         self.B = M['BBBmat']
 
         if len(args) == 3:
@@ -357,15 +359,19 @@ class Biharmonic(object):
                 S = self.S.pmat
                 A = self.A.pmat
                 B = self.B.pmat
+                A_scale *= self.A.pmat.scale
+                B_scale *= self.B.pmat.scale
+                S_scale *= self.S.pmat.scale
         elif len(args) == 6:
             S_scale = np.asscalar(args[3])
             A_scale = args[4]
             B_scale = args[5]
 
+        v = S.testfunction[0]
+        self.bc = v.bc
         if np.ndim(B_scale) > 1:
             shape = list(B_scale.shape)
             self.axis = S.axis
-            v = S.testfunction[0]
             shape[S.axis] = v.N
             self.d0 = np.zeros(shape)
             self.d1 = np.zeros(shape)
@@ -405,6 +411,8 @@ class Biharmonic(object):
     def __call__(self, u, b):
         u[:] = b
         self.PDMA_SymSolve_VC(self.d0, self.d1, self.d2, u, self.axis)
+        if self.bc is not None:
+            self.bc.set_boundary_dofs(u, True)
         return u
 
     def matvec(self, v, c):
@@ -461,7 +469,7 @@ class Helmholtz_2dirichlet(object):
         # Create transfer object to realign data in second direction
         self.T = T = matrices[0].space
         pencilA = T.forward.output_pencil
-        pencilB = pencilA.pencil(1)
+        pencilB = T.forward.input_pencil
         self.pencilB = pencilB
         self.transAB = pencilA.transfer(pencilB, 'd')
         self.u_B = np.zeros(self.transAB.subshapeB)
@@ -542,9 +550,10 @@ class Helmholtz_2dirichlet(object):
 
             B1_scale = np.zeros((ls[0].stop-ls[0].start, 1))
             B1_scale[:, 0] = self.BB.scale + 1./self.lmbda[ls[0]]
+            assert np.allclose(self.scale['AUB'], 1., 1e-8), 'Use inner(grad(v), grad(u)) or inner(v, -div(grad(u))) to get correct scaling for solver'
             A1_scale = np.ones((1, 1))
             # Create Helmholtz solver along axis=1
-            Helmy = Helmholtz(self.A1, self.B1, A1_scale, B1_scale)
+            Helmy = Helmholtz(self.A1, self.B1, A1_scale, B1_scale, local_shape=self.rhs_B.shape)
             # Map the right hand side to eigen space
             self.rhs_A = (self.V.T).dot(b)
             self.rhs_A /= self.lmbda[:, np.newaxis]
