@@ -204,7 +204,6 @@ for computing the (weighted) scalar product.
 #pylint: disable=unused-argument, not-callable, no-self-use, protected-access, too-many-public-methods, missing-docstring
 
 import importlib
-import warnings
 import sympy as sp
 import numpy as np
 from mpi4py_fft import fftw
@@ -231,38 +230,39 @@ class SpectralBase(object):
             For padding backward transform (for dealiasing)
         domain : 2-tuple of floats, optional
             The computational domain
+        dtype : data-type, optional
+            Type of input data in real physical space. Will be overloaded when
+            basis is part of a :class:`.TensorProductSpace`.
         dealias_direct : bool, optional
             If True, then set all upper 2/3 wavenumbers to zero before backward
             transform. Cannot be used if padding_factor is different than 1.
-        cordinates: 2-tuple (coordinate, position vector), optional
-        Map for curvilinear coordinatesystem.
-        The new coordinate variable in the new coordinate system is the first item.
-        Second item is a tuple for the Cartesian position vector as function of the
-        new variable in the first tuple. Example::
+        coordinates: 2-tuple (coordinate, position vector), optional
+            Map for curvilinear coordinatesystem.
+            The new coordinate variable in the new coordinate system is the first item.
+            Second item is a tuple for the Cartesian position vector as function of the
+            new variable in the first tuple. Example::
 
-            theta = sp.Symbols('x', real=True, positive=True)
-            rv = (sp.cos(theta), sp.sin(theta))
+                theta = sp.Symbols('x', real=True, positive=True)
+                rv = (sp.cos(theta), sp.sin(theta))
 
-        where theta and rv are the first and second items in the 2-tuple.
+            where theta and rv are the first and second items in the 2-tuple.
 
     """
     # pylint: disable=method-hidden, too-many-instance-attributes
 
-    def __init__(self, N, quad='', padding_factor=1, domain=(-1., 1.),
+    def __init__(self, N, quad='', padding_factor=1, domain=(-1., 1.), dtype=None,
                  dealias_direct=False, coordinates=None):
         self.N = N
         self.domain = domain
         self.quad = quad
         self.axis = 0
-        self.xfftn_fwd = None
-        self.xfftn_bck = None
         self.bc = None
         self.padding_factor = np.floor(N*padding_factor)/N
         self.dealias_direct = dealias_direct
         self._mass = None         # Mass matrix (if needed)
+        self._M = 1.0             # Normalization factor
         self._xfftn_fwd = None    # external forward transform function
         self._xfftn_bck = None    # external backward transform function
-        self._M = 1.0             # Normalization factor
         self.hi = np.ones(1, dtype=object)  # Integral measure (in addition to weight)
         coors = coordinates if coordinates is not None else ((sp.Symbol('x', real=True),),)*2
         self.coors = Coordinates(*coors)
@@ -343,7 +343,7 @@ class SpectralBase(object):
         return X
 
     def curvilinear_mesh(self, uniform=False):
-        """Return curvilinear mesh of of basis
+        """Return curvilinear mesh of basis
 
         Parameters
         ----------
@@ -582,7 +582,13 @@ class SpectralBase(object):
                 output_array
 
         """
-        raise NotImplementedError
+        x = np.atleast_1d(x)
+        if output_array is None:
+            output_array = np.zeros(x.shape, dtype=self.dtype)
+        X = sp.symbols('x', real=True)
+        f = self.sympy_basis(i, X)
+        output_array[:] = sp.lambdify(X, f)(x)
+        return output_array
 
     def evaluate_basis_all(self, x=None, argument=0):
         """Evaluate basis at ``x`` or all quadrature points
@@ -623,13 +629,13 @@ class SpectralBase(object):
                 output_array
 
         """
-        warnings.warn('Using slow sympy evaluate_basis_derivative')
+        #warnings.warn('Using slow sympy evaluate_basis_derivative')
         if x is None:
             x = self.mesh(False, False)
         if output_array is None:
-            output_array = np.zeros(x.shape)
+            output_array = np.zeros(x.shape, dtype=self.dtype)
         x = np.atleast_1d(x)
-        X = sp.symbols('x')
+        X = sp.symbols('x', real=True)
         basis = self.sympy_basis(i=i, x=X).diff(X, k)
         output_array[:] = sp.lambdify(X, basis, 'numpy')(x)
         return output_array
@@ -923,7 +929,7 @@ class SpectralBase(object):
 
         """
         if output_array is None:
-            output_array = np.zeros(x.shape, dtype=self.forward.input_array.dtype)
+            output_array = np.zeros(x.shape, dtype=self.dtype)
         x = self.map_reference_domain(x)
         return self.vandermonde_evaluate_expansion(x, u, output_array)
 
@@ -1003,6 +1009,11 @@ class SpectralBase(object):
             return 1
         return R/L
 
+    @property
+    def dtype(self):
+        """Return datatype basis is planned for"""
+        return self.forward.input_array.dtype
+
     @property
     def dimensions(self):
         """Return the dimensions (the number of bases) of the
@@ -1038,6 +1049,10 @@ class SpectralBase(object):
     def is_orthogonal(self):
         return False
 
+    @property
+    def is_composite_space(self):
+        return 0
+
     @property
     def has_nonhomogeneous_bcs(self):
         return False
@@ -1101,6 +1116,14 @@ class SpectralBase(object):
     def __hash__(self):
         return hash((self.N, self.quad, self.family()))
 
+    def get_bcmass_matrix(self, hi=1):
+        msx = 'xyzrs'[self.axis]
+        dV = split(hi)
+        assert len(dV) == 1
+        dv = dV[0]
+        msi = dv[msx]
+        return inner_product((self, 0), (self.get_bc_basis(), 0), msi)
+
     def get_measured_weights(self, N=None, measure=1):
         """Return weights times `measure`
 
@@ -1179,6 +1202,7 @@ class SpectralBase(object):
         return self.__class__(self.N,
                               quad=self.quad,
                               domain=self.domain,
+                              dtype=self.dtype,
                               padding_factor=padding_factor,
                               dealias_direct=dealias_direct,
                               coordinates=self.coors.coordinates)
@@ -1199,6 +1223,7 @@ class SpectralBase(object):
         return self.__class__(N,
                               quad=self.quad,
                               domain=self.domain,
+                              dtype=self.dtype,
                               padding_factor=self.padding_factor,
                               dealias_direct=self.dealias_direct,
                               coordinates=self.coors.coordinates)
@@ -1297,9 +1322,13 @@ class MixedBasis(object):
         return self.bases[0].dimensions
 
     @property
-    def rank(self):
+    def is_composite_space(self):
         return 1
 
+    @property
+    def rank(self):
+        return None
+
     @property
     def dimensions(self):
         """Return dimension of scalar space"""
@@ -1328,6 +1357,7 @@ class VectorBasisTransform(object):
             output_array[i] = transform(input_array[i], output_array[i], **kw)
         return output_array
 
+
 class islicedict(dict):
     """Return a tuple of slices, broadcasted to ``dimensions`` number of
     dimensions, and with integer ``a`` along ``axis``.
