@@ -3,12 +3,11 @@ Module for implementation of the :class:`.TensorProductSpace` class and
 related methods.
 """
 from numbers import Number
-import warnings
 import functools
 import sympy as sp
 import numpy as np
 from shenfun.fourier.bases import R2CBasis, C2CBasis
-from shenfun.utilities import apply_mask, split
+from shenfun.utilities import apply_mask
 from shenfun.forms.arguments import Function, Array
 from shenfun.optimization.cython import evaluate
 from shenfun.spectralbase import slicedict, islicedict, SpectralBase
@@ -134,7 +133,7 @@ class TensorProductSpace(PFFT):
         Pencil distribution in spectral space. This is primarily intended
         for a padded space, where the spectral distribution must be
         equal to the non-padded space.
-    cordinates: two tuples, optional
+    coordinates: two tuples, optional
         Map for curvilinear coordinatesystem.
         First tuple are the coordinate variables in the new coordinate system
         Second tuple are the Cartesian coordinates as functions of the variables
@@ -160,6 +159,7 @@ class TensorProductSpace(PFFT):
         self.coors = Coordinates(*coors)
         self.hi = self.coors.get_scaling_factors()
         shape = list(self.global_shape())
+        self.axes = axes
         assert shape
         assert min(shape) > 0
 
@@ -193,7 +193,6 @@ class TensorProductSpace(PFFT):
         dtype = np.dtype(dtype)
         assert dtype.char in 'fdgFDG'
 
-        self.axes = axes
         self.xfftn = []
         self.transfer = []
         self.pencil = [None, None]
@@ -203,6 +202,7 @@ class TensorProductSpace(PFFT):
             base.sl = slicedict(axis=axis, dimensions=len(self.bases))
             base.si = islicedict(axis=axis, dimensions=len(self.bases))
 
+        self.axes = axes
         if not backward_from_pencil:
             if isinstance(bases[axes[-1][-1]], C2CBasis):
                 assert np.dtype(dtype).char in 'FDG'
@@ -385,7 +385,12 @@ class TensorProductSpace(PFFT):
         padded_bases = [base.get_dealiased(padding_factor=padding_factor[axis],
                                            dealias_direct=dealias_direct)
                         for axis, base in enumerate(self.bases)]
-        return TensorProductSpace(self.comm, padded_bases,
+        # Need the correct order of the transforms in case reversed somehow
+        axes = []
+        for ax in self.axes:
+            for ai in ax:
+                axes.append(ai)
+        return TensorProductSpace(self.comm, padded_bases, axes=tuple(axes),
                                   dtype=self.forward.output_array.dtype,
                                   backward_from_pencil=self.forward.output_pencil,
                                   coordinates=self.coors.coordinates)
@@ -397,7 +402,8 @@ class TensorProductSpace(PFFT):
             assert len(N) == len(self)
         refined_bases = [base.get_refined(N[axis])
                          for axis, base in enumerate(self.bases)]
-        return TensorProductSpace(self.comm, refined_bases, axes=self.axes,
+        return TensorProductSpace(self.subcomm, refined_bases, axes=self.axes,
+                                  dtype=self.forward.input_array.dtype,
                                   coordinates=self.coors.coordinates)
 
     def convolve(self, a_hat, b_hat, ab_hat):
@@ -862,7 +868,13 @@ class TensorProductSpace(PFFT):
         ortho = []
         for base in self.bases:
             ortho.append(base.get_orthogonal())
-        return TensorProductSpace(self.subcomm, ortho, axes=self.axes)
+        return TensorProductSpace(self.subcomm, ortho, axes=self.axes,
+                                  dtype=self.forward.input_array.dtype,
+                                  coordinates=self.coors.coordinates)
+
+    @property
+    def is_composite_space(self):
+        return 0
 
     def __getitem__(self, i):
         """Return instance of base i
@@ -899,10 +911,13 @@ class TensorProductSpace(PFFT):
 class MixedTensorProductSpace(object):
     """Class for composite tensorproductspaces.
 
+    The mixed spaces are Cartesian products of TensorproductSpaces or
+    other MixedTensorProductSpaces.
+
     Parameters
     ----------
     spaces : list
-        List of TensorProductSpaces
+        List of spaces
     """
 
     def __init__(self, spaces):
@@ -912,6 +927,10 @@ class MixedTensorProductSpace(object):
         self.backward_uniform = VectorTransform([space.backward_uniform for space in spaces])
         self.scalar_product = VectorTransform([space.scalar_product for space in spaces])
 
+    @property
+    def is_composite_space(self):
+        return 1
+
     def eval(self, points, coefficients, output_array=None, method=0):
         """Evaluate Function at points, given expansion coefficients
 
@@ -929,7 +948,6 @@ class MixedTensorProductSpace(object):
             The final, method = 2, is a python implementation used only
             for verification.
         """
-
         if output_array is None:
             output_array = np.zeros((len(self.flatten()), points.shape[-1]), dtype=self.forward.input_array.dtype)
         for i, space in enumerate(self.flatten()):
@@ -974,7 +992,7 @@ class MixedTensorProductSpace(object):
     @property
     def rank(self):
         """Return rank of space"""
-        return 1
+        return None
 
     def dim(self):
         """Return dimension of ``self`` (degrees of freedom)"""
@@ -1078,6 +1096,12 @@ class MixedTensorProductSpace(object):
     def get_refined(self, N):
         raise NotImplementedError
 
+    def get_orthogonal(self):
+        raise NotImplementedError
+
+    def get_dealiased(self):
+        raise NotImplementedError
+
     def __getitem__(self, i):
         return self.spaces[i]
 
@@ -1094,20 +1118,20 @@ class VectorTensorProductSpace(MixedTensorProductSpace):
     """A special :class:`.MixedTensorProductSpace` where the number of spaces
     must equal the geometrical dimension of the problem.
 
-    For example, a TensorProductSpace created by a Cartesian product of 2 1D
+    For example, a TensorProductSpace created by a tensorproduct of 2 1D
     bases, will have vectors of length 2. A TensorProductSpace created from 3
     1D bases will have vectors of length 3.
 
     Parameters
     ----------
-    space : :class:`.TensorProductSpace`
-        Space to create vector from
+    space : :class:`.TensorProductSpace` or list of ndim :class:`.TensorProductSpace`s
+        Spaces to create vector from
 
     """
 
     def __init__(self, space):
         if isinstance(space, list):
-            warnings.warn("Use only the TensorProductSpace as argument", DeprecationWarning)
+            assert len(space) == space[0].dimensions
             spaces = space
         else:
             spaces = [space]*space.dimensions
@@ -1139,10 +1163,19 @@ class VectorTensorProductSpace(MixedTensorProductSpace):
         return s
 
     def get_refined(self, N):
-        return VectorTensorProductSpace(self.spaces[0].get_refined(N))
+        if np.all([s == self.spaces[0] for s in self.spaces[1:]]):
+            return VectorTensorProductSpace(self.spaces[0].get_refined(N))
+        return VectorTensorProductSpace([s.get_refined(N) for s in self.spaces])
 
     def get_dealiased(self, padding_factor=1.5, dealias_direct=False):
-        return VectorTensorProductSpace(self.spaces[0].get_dealiased(padding_factor, dealias_direct))
+        if np.all([s == self.spaces[0] for s in self.spaces[1:]]):
+            return VectorTensorProductSpace(self.spaces[0].get_dealiased(padding_factor, dealias_direct))
+        return VectorTensorProductSpace([s.get_dealiased(padding_factor, dealias_direct) for s in self.spaces])
+
+    def get_orthogonal(self):
+        if np.all([s == self.spaces[0] for s in self.spaces[1:]]):
+            return VectorTensorProductSpace(self.spaces[0].get_orthogonal())
+        return VectorTensorProductSpace([s.get_orthogonal() for s in self.spaces])
 
 class VectorTransform(object):
 
@@ -1266,7 +1299,7 @@ class BoundaryValues(object):
             self.bcs_final[:] = self.bcs
 
     def update_bcs_time(self, time):
-        tt = sp.symbols('t')
+        tt = sp.symbols('t', real=True)
         update_time = False
         for i, bci in enumerate(self.bc):
             if isinstance(bci, sp.Expr):
@@ -1327,8 +1360,9 @@ class BoundaryValues(object):
                 if isinstance(bci, sp.Expr):
                     X = T.local_mesh(True)
                     for sym in bci.free_symbols:
-                        if str(sym) == 't':
-                            bci = bci.subs({'t': self.bc_time})
+                        tt = sp.symbols('t', real=True)
+                        if sym == tt:
+                            bci = bci.subs(tt, self.bc_time)
                     sym0 = bci.free_symbols
                     lbci = sp.lambdify(sym0, bci, 'numpy')
                     Yi = []
@@ -1431,15 +1465,10 @@ class BoundaryValues(object):
             Add boundary values to this Function
 
         """
-        B = self.base.get_bc_basis()
-        M = B.addmass_matrix()
-        sl = B.slice()
-        if u.ndim > 1:
-            sl = self.base.sl[sl]
-
-        for i, row in enumerate(M):
-            for j in range(len(self.bc)):
-                u[self.base.si[i]] -= row[j]*self.bcs[j]
+        coors = self.tensorproductspace.coors if self.tensorproductspace else self.base.coors
+        M = self.base.get_bcmass_matrix(coors.hi.prod())
+        w0 = np.zeros_like(u)
+        u -= M.matvec(u, w0, axis=self.base.axis)
 
     def set_boundary_dofs(self, u, final=False):
         """Apply boundary condition after solving, fixing boundary dofs
