@@ -114,7 +114,7 @@ class ETD(IntegratorBase):
         self.params['dt'] = dt
         L = self.LinearRHS(**self.params)
         if isinstance(L, TPMatrix):
-            assert L.all_identity()
+            assert L.isidentity()
             L = L.scale
         L = np.atleast_1d(L)
         hL = L*dt
@@ -194,12 +194,12 @@ class ETDRK4(IntegratorBase):
         self.params['dt'] = dt
         L = self.LinearRHS(**self.params)
         if isinstance(L, TPMatrix):
-            assert L.all_identity()
+            assert L.isidentity()
             L = L.scale
         L = np.atleast_1d(L)
         hL = L*dt
-        ehL = self.ehL = np.exp(hL)
-        ehL_h = self.ehL_h = np.exp(hL/2.)
+        self.ehL = np.exp(hL)
+        self.ehL_h = np.exp(hL/2.)
 
         M = 50
         psi = self.psi = np.zeros((4,) + hL.shape, dtype=np.float)
